import sys
import os
from os.path import dirname
import re
import json

from mrjob.job import MRJob
from mrjob.step import MRStep

import Geohash


try:
    # when running on EMR a geotweet package will be loaded onto PYTHON PATH
    from geotweet.mapreduce.utils.reader import FileReader
    from geotweet.mapreduce.utils.words import WordExtractor
    from geotweet.mapreduce.utils.lookup import CachedCountyLookup
except ImportError:
    # runing locally
    from utils.reader import FileReader
    from utils.words import WordExtractor
    from utils.lookup import CachedCountyLookup


"""
https://en.wikipedia.org/wiki/Geohash

geohash
precision   width   height
4           39.1km  19.5km
5           4.9km   4.9km
6           1.2km   609.4m
7           152.9m  152.4m
8           38.2m   19m
"""
GEOHASH_PRECISION = 7
MIN_WORD_COUNT = 5      # ignore low occurences


class MRStateCountyWordCount(MRJob):
    """
    Count word occurences for US tweets by entire county, by State and County
    
    A geojson file of US counties is downloaded from an S3 bucket. A RTree index
    is built using the bounding box of each county, and is used for determining
    State and County for each tweet.
    
    """

    def steps(self):
        return [
            MRStep(
                mapper_init=self.mapper_init,
                mapper=self.mapper,
                combiner=self.combiner,
                reducer=self.reducer
            )
        ]

    def mapper_init(self):
        """ Download counties geojson from S3 and build spatial index and cache """
        self.counties = CachedCountyLookup()
        self.extractor = WordExtractor()
    
    def mapper(self, _, line):
        data = json.loads(line)
        # ignore HR geo-tweets for job postings
        if data['description'] and self.hr_filter(data['description']):
            return
        # convert coordinates to geohash
        lat = data['lonlat'][1]
        lon = data['lonlat'][0]
        geohash = Geohash.encode(lat, lon, precision=GEOHASH_PRECISION)
        # spatial lookup for state and county
        state, county = self.counties.get(geohash) 
        if not state or not county:
            return
        # count words
        for word in self.extractor.run(data['text']):
            yield (word, ), 1
            yield (word, state), 1
            yield (word, state, county), 1
    
    def hr_filter(self, text):
        """ check if description of twitter using contains job related key words """
        expr = "|".join(["(job)", "(hiring)", "(career)"])
        return re.findall(expr, text)
  
    def combiner(self, key, values):
        yield key, sum(values)
    
    def reducer(self, key, values):
        total = int(sum(values))
        if total < MIN_WORD_COUNT:
            return
        word = state = county = None
        word = key[0]
        if len(key) >= 2:
            state = key[1]
        if len(key) >= 3:
            county = key[2]
        yield (word, state, county), total


if __name__ == '__main__':
    MRStateCountyWordCount.run()



from mrjob.job import MRJob


class MRWordFrequencyCount(MRJob):

    def mapper(self, _, line):
        yield "chars", len(line)
        yield "words", len(line.split())
        yield "lines", 1

    def reducer(self, key, values):
        yield key, sum(values)


if __name__ == '__main__':
    MRWordFrequencyCount.run()



import os
import shutil
import errno

from load import GeoJSONLoader
from mongo import MongoGeo

import logging
logger = logging.getLogger(__name__)


class GeoMongo(object):
    
    def __init__(self, args):
        self.source = args.file
        self.mongo = args.mongo
        self.db = args.db
        self.collection = args.collection

    def run(self):
        """ Top level runner to load State and County  GeoJSON files into Mongo DB """
        logger.info("Starting GeoJSON MongoDB loading process.")
        mongo = dict(uri=self.mongo, db=self.db, collection=self.collection)
        self.load(self.source, **mongo)
        logger.info("Finished loading {0} into MongoDB".format(self.source))

    def load(self, geojson, uri=None, db=None, collection=None):
        """ Load geojson file into mongodb instance """
        logger.info("Mongo URI: {0}".format(uri))
        logger.info("Mongo DB: {0}".format(db))
        logger.info("Mongo Collection: {0}".format(collection))
        logger.info("Geojson File to be loaded: {0}".format(geojson))
        mongo = MongoGeo(db=db, collection=collection, uri=uri)
        GeoJSONLoader().load(geojson, mongo.insert)  



import json


class GeoJSONLoader(object):

    def load(self, json_file, load_func):
        with open(json_file, 'r') as f:
            data = json.loads(f.read().decode('latin-1').encode('utf-8'))
            for feature in data['features']:
                load_func(feature)



import sys
import os
from os.path import dirname
import re
import json

from mrjob.job import MRJob
from mrjob.step import MRStep


try:
    # when running on EMR a geotweet package will be loaded onto PYTHON PATH
    from geotweet.mapreduce.utils.words import WordExtractor
    from geotweet.geomongo.mongo import MongoGeo
except ImportError:
    # runing locally
    from utils.words import WordExtractor
    parent = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    sys.path.insert(0, parent) 
    from geomongo.mongo import MongoGeo


DB = "geotweet"
COLLECTION = "geotweet"
MIN_WORD_COUNT = 2
METERS_PER_MILE = 1609
METRO_DISTANCE = 50 * METERS_PER_MILE


class MRMetroMongoWordCount(MRJob):
    """
    Map Reduce job that counts word occurences for each US Metro Area

    Requires a running MongoDB instance with us_metro_areas.geojson loaded

    # Command to load metro area. Must name the collection 'metro'
    `geoloader geojson /path/to/us_metro_areas.geojson metro`
    
    Mapper:

        1. Ignore tweets that appear to be from HR accounts about jobs and hiring
        2. Run MongoDB query to lookup nearest metro area based on coordinates
        3. Tokenize tweet in individual words
        4. For each word output the tuple: (('metro area', 'word'), 1)

    Reducer:

        1. Sum count for each ('metro area', 'word') key
        2. Insert record into MongoDB with word count for that metro area
    
    """

    def steps(self):
        return [
            MRStep(
                mapper_init=self.mapper_init,
                mapper=self.mapper,
                combiner=self.combiner,
                reducer_init=self.reducer_init,
                reducer=self.reducer
            )
        ]
    
    def mapper_init(self):
        self.mongo = MongoGeo(db='geotweet', collection='metro')
        self.extractor = WordExtractor()
   
    def mapper(self, _, line):
        data = json.loads(line)
        # ignore HR geo-tweets for job postings
        expr = "|".join(["(job)", "(hiring)", "(career)"])
        if data['description'] and re.findall(expr, data['description']):
            return
        # lookup nearest metro area
        lat = data['lonlat'][1]
        lon = data['lonlat'][0]
        near = self.mongo.near([lon, lat], distance=METRO_DISTANCE)
        try:
            nearest = self.mongo.find(query=near, limit=1).next()
            metro = nearest['properties']['NAME10']
        except StopIteration:
            return
        # count each word
        for word in self.extractor.run(data['text']):
            yield (metro, word), 1
            
    def combiner(self, key, value):
        yield key, sum(value)
    
    def reducer_init(self):
        self.mongo = MongoGeo(db=DB, collection=COLLECTION)
   
    def reducer(self, key, values):
        total = int(sum(values))
        if total < MIN_WORD_COUNT:
            return
        metro, word = key
        self.mongo.insert(dict(
            metro_area=metro,
            word=word,
            count=total
        ))
        yield metro, (total, word) 


if __name__ == '__main__':
    MRMetroMongoWordCount.run()
